@@ -32,6 +32,7 @@
 #include "pango-fontmap.h"
 #include "pango-utils.h"
 #include "pangowin32-private.h"
+#include "modules.h"
 
 typedef struct _PangoWin32Family       PangoWin32Family;
 typedef struct _PangoWin32SizeInfo     PangoWin32SizeInfo;
@@ -184,6 +185,7 @@ pango_win32_font_map_class_init (PangoWin32FontMapClass *class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (class);
   PangoFontMapClass *fontmap_class = PANGO_FONT_MAP_CLASS (class);
+  int i;
 
   class->find_font = pango_win32_font_map_real_find_font;
   object_class->finalize = pango_win32_font_map_finalize;
@@ -192,6 +194,9 @@ pango_win32_font_map_class_init (PangoWin32FontMapClass *class)
   fontmap_class->shape_engine_type = PANGO_RENDER_TYPE_WIN32;
   
   pango_win32_get_dc ();
+
+  for (i = 0; _pango_included_win32_modules[i].list; i++)
+    pango_module_register (&_pango_included_win32_modules[i]);
 }
 
 static PangoWin32FontMap *default_fontmap = NULL;
