@@ -92,6 +92,7 @@ static void                  pango_x_font_get_glyph_extents (PangoFont        *f
 							     PangoRectangle   *logical_rect);
 static PangoFontMetrics *    pango_x_font_get_metrics       (PangoFont        *font,
 							     PangoLanguage    *language);
+static PangoFontMap *        pango_x_font_get_font_map      (PangoFont        *font);
 
 static PangoXSubfontInfo * pango_x_find_subfont    (PangoFont          *font,
 						    PangoXSubfont       subfont_index);
@@ -283,6 +284,7 @@ pango_x_font_class_init (PangoXFontClass *class)
   font_class->find_shaper = pango_x_font_find_shaper;
   font_class->get_glyph_extents = pango_x_font_get_glyph_extents;
   font_class->get_metrics = pango_x_font_get_metrics;
+  font_class->get_font_map = pango_x_font_get_font_map;
 }
 
 PangoXFont *
@@ -850,7 +852,7 @@ get_font_metrics_from_subfonts (PangoFont        *font,
     metrics->approximate_char_width = 10 * PANGO_SCALE;
 }
 
-void
+static void
 get_subfonts_foreach (PangoFont      *font,
 		      PangoGlyphInfo *glyph_info,
 		      gpointer        data)
@@ -882,7 +884,7 @@ get_font_metrics_from_string (PangoFont        *font,
   g_slist_free (subfonts);
 }
 
-void
+static void
 average_width_foreach (PangoFont      *font,
 		       PangoGlyphInfo *glyph_info,
 		       gpointer        data)
@@ -944,6 +946,14 @@ pango_x_font_get_metrics (PangoFont        *font,
   return pango_font_metrics_ref (info->metrics);
 }
 
+static PangoFontMap *
+pango_x_font_get_font_map (PangoFont *font)
+{
+  PangoXFont *xfont = (PangoXFont *)font;
+
+  return xfont->fontmap;
+}
+
 /* Compare the tail of a to b */
 static gboolean
 match_end (char *a, char *b)
@@ -1500,6 +1510,27 @@ pango_x_render_layout_line (Display          *display,
 		     x + (x_off + ink_rect.x) / PANGO_SCALE - 1, y + 2,
 		     x + (x_off + ink_rect.x + ink_rect.width) / PANGO_SCALE, y + 2);
 	  break;
+	case PANGO_UNDERLINE_ERROR:
+          {
+            int point_x;
+            int counter = 0;
+	    int end_x = x + (x_off + ink_rect.x + ink_rect.width) / PANGO_SCALE;
+
+            for (point_x = x + PANGO_PIXELS (x_off + ink_rect.x) - 1;
+                 point_x <= end_x;
+                 point_x += 2)
+	      {
+		if (counter)
+		  XDrawLine (display, drawable, gc,
+			     point_x, y + 2, MIN (point_x + 1, end_x), y + 2);
+		else
+		  XDrawLine (display, drawable, gc,
+			     point_x, y + 3, MIN (point_x + 1, end_x), y + 3);
+		
+		counter = (counter + 1) % 2;
+	      }
+          }
+	  break;
 	case PANGO_UNDERLINE_LOW:
 	  XDrawLine (display, drawable, fg_gc,
 		     x + (x_off + ink_rect.x) / PANGO_SCALE - 1, y + (ink_rect.y + ink_rect.height) / PANGO_SCALE + 2,
